/*
Copyright 2020 The KubeEdge Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
// Code generated by client-gen. DO NOT EDIT.

package v1alpha1

import (
	"time"

	v1alpha1 "github.com/kubeedge/kubeedge/cloud/pkg/apis/reliablesyncs/v1alpha1"
	scheme "github.com/kubeedge/kubeedge/cloud/pkg/client/clientset/versioned/scheme"
	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// ClusterObjectSyncsGetter has a method to return a ClusterObjectSyncInterface.
// A group's client should implement this interface.
type ClusterObjectSyncsGetter interface {
	ClusterObjectSyncs() ClusterObjectSyncInterface
}

// ClusterObjectSyncInterface has methods to work with ClusterObjectSync resources.
type ClusterObjectSyncInterface interface {
	Create(*v1alpha1.ClusterObjectSync) (*v1alpha1.ClusterObjectSync, error)
	Update(*v1alpha1.ClusterObjectSync) (*v1alpha1.ClusterObjectSync, error)
	UpdateStatus(*v1alpha1.ClusterObjectSync) (*v1alpha1.ClusterObjectSync, error)
	Delete(name string, options *v1.DeleteOptions) error
	DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error
	Get(name string, options v1.GetOptions) (*v1alpha1.ClusterObjectSync, error)
	List(opts v1.ListOptions) (*v1alpha1.ClusterObjectSyncList, error)
	Watch(opts v1.ListOptions) (watch.Interface, error)
	Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha1.ClusterObjectSync, err error)
	ClusterObjectSyncExpansion
}

// clusterObjectSyncs implements ClusterObjectSyncInterface
type clusterObjectSyncs struct {
	client rest.Interface
}

// newClusterObjectSyncs returns a ClusterObjectSyncs
func newClusterObjectSyncs(c *ReliablesyncsV1alpha1Client) *clusterObjectSyncs {
	return &clusterObjectSyncs{
		client: c.RESTClient(),
	}
}

// Get takes name of the clusterObjectSync, and returns the corresponding clusterObjectSync object, and an error if there is any.
func (c *clusterObjectSyncs) Get(name string, options v1.GetOptions) (result *v1alpha1.ClusterObjectSync, err error) {
	result = &v1alpha1.ClusterObjectSync{}
	err = c.client.Get().
		Resource("clusterobjectsyncs").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do().
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of ClusterObjectSyncs that match those selectors.
func (c *clusterObjectSyncs) List(opts v1.ListOptions) (result *v1alpha1.ClusterObjectSyncList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1alpha1.ClusterObjectSyncList{}
	err = c.client.Get().
		Resource("clusterobjectsyncs").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do().
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested clusterObjectSyncs.
func (c *clusterObjectSyncs) Watch(opts v1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Resource("clusterobjectsyncs").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch()
}

// Create takes the representation of a clusterObjectSync and creates it.  Returns the server's representation of the clusterObjectSync, and an error, if there is any.
func (c *clusterObjectSyncs) Create(clusterObjectSync *v1alpha1.ClusterObjectSync) (result *v1alpha1.ClusterObjectSync, err error) {
	result = &v1alpha1.ClusterObjectSync{}
	err = c.client.Post().
		Resource("clusterobjectsyncs").
		Body(clusterObjectSync).
		Do().
		Into(result)
	return
}

// Update takes the representation of a clusterObjectSync and updates it. Returns the server's representation of the clusterObjectSync, and an error, if there is any.
func (c *clusterObjectSyncs) Update(clusterObjectSync *v1alpha1.ClusterObjectSync) (result *v1alpha1.ClusterObjectSync, err error) {
	result = &v1alpha1.ClusterObjectSync{}
	err = c.client.Put().
		Resource("clusterobjectsyncs").
		Name(clusterObjectSync.Name).
		Body(clusterObjectSync).
		Do().
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().

func (c *clusterObjectSyncs) UpdateStatus(clusterObjectSync *v1alpha1.ClusterObjectSync) (result *v1alpha1.ClusterObjectSync, err error) {
	result = &v1alpha1.ClusterObjectSync{}
	err = c.client.Put().
		Resource("clusterobjectsyncs").
		Name(clusterObjectSync.Name).
		SubResource("status").
		Body(clusterObjectSync).
		Do().
		Into(result)
	return
}

// Delete takes name of the clusterObjectSync and deletes it. Returns an error if one occurs.
func (c *clusterObjectSyncs) Delete(name string, options *v1.DeleteOptions) error {
	return c.client.Delete().
		Resource("clusterobjectsyncs").
		Name(name).
		Body(options).
		Do().
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *clusterObjectSyncs) DeleteCollection(options *v1.DeleteOptions, listOptions v1.ListOptions) error {
	var timeout time.Duration
	if listOptions.TimeoutSeconds != nil {
		timeout = time.Duration(*listOptions.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Resource("clusterobjectsyncs").
		VersionedParams(&listOptions, scheme.ParameterCodec).
		Timeout(timeout).
		Body(options).
		Do().
		Error()
}

// Patch applies the patch and returns the patched clusterObjectSync.
func (c *clusterObjectSyncs) Patch(name string, pt types.PatchType, data []byte, subresources ...string) (result *v1alpha1.ClusterObjectSync, err error) {
	result = &v1alpha1.ClusterObjectSync{}
	err = c.client.Patch(pt).
		Resource("clusterobjectsyncs").
		SubResource(subresources...).
		Name(name).
		Body(data).
		Do().
		Into(result)
	return
}
